import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";

class BorderLabel extends StatelessWidget {
  final Widget child;
  final String label;
  final double? height;
  final double? width;
  final Function()? onPressed;
  final Function()? onCopied;

  final List<Widget> actions;

  const BorderLabel({
    Key? key,
    required this.label,
    required this.child,
    this.actions = const [],
    this.onPressed,
    this.onCopied,
    this.height,
    this.width,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: const EdgeInsets.symmetric(vertical: 6),
      child: LayoutBuilder(
        builder: (context, constraints) {
          final double _width = width ??
              (constraints.maxWidth != double.infinity
                  ? constraints.maxWidth
                  : 400);
          final double _height = height ?? 100;
          double padding = 1.5;
          return GestureDetector(
            onTap: onPressed,
            child: Container(
              width: _width,
              height: _height,
              decoration: const BoxDecoration(
                borderRadius: BorderRadius.all(
                  Radius.circular(5),
                ),
              ),
              child: Container(
                width: constraints.maxWidth,
                decoration: BoxDecoration(
                  border: Border.all(
                    width: .3,
                  ),
                  borderRadius: const BorderRadius.all(
                    Radius.circular(5),
                  ),
                ),
                padding: EdgeInsets.all(padding),
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    SizedBox(
                      width: _width,
                      height: 28.5,
                      child: Column(
                        children: [
                          const SizedBox(
                            height: 1,
                          ),
                          SizedBox(
                            width: _width,
                            child: Row(
                              crossAxisAlignment: CrossAxisAlignment.center,
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                Expanded(
                                  child: Padding(
                                    padding: EdgeInsets.all(padding),
                                    child: Align(
                                      alignment: Alignment.centerLeft,
                                      child: SelectableText(
                                        label,
                                        maxLines: 1,
                                        style: const TextStyle(
                                          fontSize: 15,
                                          overflow: TextOverflow.fade,
                                        ),
                                      ),
                                    ),
                                  ),
                                ),
                                ...[
                                  ...actions,
                                  if (onCopied != null)
                                    PhysicalModel(
                                      color: Colors.grey,
                                      elevation: 5,
                                      shape: BoxShape.circle,
                                      child: CircleAvatar(
                                        child: InkWell(
                                          child: ThemedIcons.copy(
                                            iconStyle: IconStyle(
                                              size: 20,
                                            ),
                                          ),
                                          onTap: onCopied!,
                                        ),
                                      ),
                                    ),
                                  if (onPressed != null)
                                    PhysicalModel(
                                      color: Colors.grey,
                                      elevation: 5,
                                      shape: BoxShape.circle,
                                      child: CircleAvatar(
                                        child: InkWell(
                                          child: ThemedIcons.moreHoriz(
                                            iconStyle: IconStyle(
                                              size: 20,
                                            ),
                                          ),
                                          onTap: onPressed!,
                                        ),
                                      ),
                                    ),
                                ]
                                    .map(
                                      (e) => SizedBox(
                                        height: 26.5,
                                        width: 26.5,
                                        child: Center(
                                          child: SizedBox(
                                            height: 26,
                                            width: 26,
                                            child: Center(child: e),
                                          ),
                                        ),
                                      ),
                                    )
                                    .toList(),
                              ],
                            ),
                          ),
                          const SizedBox(
                            height: 1,
                          ),
                        ],
                      ),
                    ),
                    const Divider(
                      height: 1,
                    ),
                    Expanded(
                      child: Padding(
                        padding: EdgeInsets.all(padding),
                        child: Align(
                          alignment: Alignment.centerLeft,
                          child: child,
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ),
          );
        },
      ),
    );
  }
}
