import "dart:async";
import "dart:typed_data";
import "package:carousel_slider/carousel_slider.dart";
import 'package:image_picker/image_picker.dart';
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/config/config.dart';
import "package:verttice_ui/src/const/box_sizes.dart";
import "package:verttice_ui/src/const/icons.dart";
import "package:file_picker/file_picker.dart";
import "package:flutter/material.dart";
import "package:mime/mime.dart";
import 'package:verttice_ui/src/widgets/files/image_viewer.dart';
import 'package:verttice_ui/src/widgets/themed_app_bar.dart';
import 'package:verttice_ui/src/widgets/themed_progress.dart';
import 'package:verttice_ui/src/widgets/themed_text.dart';
import 'package:verttice_ui/src/widgets/themed_text_button.dart';
import 'package:verttice_ui/src/widgets/themed_toast.dart';

class UploadFile extends StatelessWidget {
  final Function(List<FileModel> fileData) filesDataCallback;
  final String path;
  final String? buttonText;
  final bool multi;
  final Widget? altWidget;

  const UploadFile({
    Key? key,
    required this.path,
    required this.filesDataCallback,
    this.buttonText,
    this.multi = false,
    this.altWidget,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (altWidget != null) {
      return InkWell(
        child: altWidget,
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => UplaodFilesAction(
                path: path,
                filesDataCallback: filesDataCallback,
                multi: multi,
              ),
            ),
          );
        },
      );
    }
    return ThemedTextButton(
      buttonText ?? "Upload Files",
      onPressed: () {
        Navigator.push(
          context,
          MaterialPageRoute(
            builder: (context) => UplaodFilesAction(
              path: path,
              filesDataCallback: filesDataCallback,
              multi: multi,
            ),
          ),
        );
      },
    );
  }
}

class UplaodFilesAction extends StatefulWidget {
  final Function(List<FileModel> fileData) filesDataCallback;
  final String path;
  final bool multi;

  const UplaodFilesAction({
    Key? key,
    required this.path,
    required this.filesDataCallback,
    this.multi = false,
  }) : super(key: key);

  @override
  _UplaodFilesActionState createState() => _UplaodFilesActionState();
}

class _UplaodFilesActionState extends State<UplaodFilesAction> {
  int _current = 0;
  final CarouselController _controller = CarouselController();
  final fileTypes = [
    "jpg",
    "png",
    "jpeg",
    "giff",
    "gif",
    "ico",
    "pdf",
    // "m1v",
    // "ogg",
    // "webm",
    "mp4",
    "m4a",
    "mov",
    // "m4p",
    // "m4b",
    // "m4r",
    // "m4v",
    // "avi",
    // "asf",
    "wma",
    "wmv",
  ];

  bool _loading = false;
  List<PlatformFile>? _files;

  Future<void> _getFiles() async {
    try {
      setState(() {
        _loading = true;
      });
      List<PlatformFile>? files;

      final imageFile = await FilePicker.platform.pickFiles(
        type: FileType.custom,
        allowedExtensions: fileTypes,
        allowMultiple: widget.multi,
        withData: true,
      );
      files = imageFile?.files;

      if (files != null && files.isNotEmpty) {
        setState(() {
          _files = _files == null ? files : [..._files!, ...files!];
        });
      }
    } catch (e) {
      ThemedToast.error(context: context, message: "Failed to get files");
    } finally {
      setState(() {
        _loading = false;
      });
    }
  }

  Future<void> _uploadAndRetutnImageAndUrl(BuildContext context) async {
    setState(() {
      _loading = true;
    });
    try {
      if (_files!.length > 5) {
        ThemedToast.error(
          context: context,
          message: "Can't upload more than 5 files at once, please try again",
        );
      } else {
        List<FileModel> allImageRes = [];
        final response = await VertticeUIConfig.instance.uploadFiles(
            context: context,
            files: _files!.map((e) {
              final String fileID = IDs.l16();
              Uint8List? file = e.bytes;
              String? mimeType =
                  lookupMimeType(e.name, headerBytes: file) ?? "";

              return UploadFileDataModel(
                resize: e.size > 4200000 ? 720 : null,
                fileID: fileID,
                file: file,
                mimeType: mimeType,
                path: "${widget.path}/$fileID",
                name: e.name,
                size: e.size,
              );
            }).toList());

        if (response != null) {
          allImageRes = response;
        } else {
          throw "Res err";
        }

        widget.filesDataCallback(allImageRes);
      }
    } catch (e) {
      ThemedToast.error(
        context: context,
        message: "Failed to upload files, please try again",
      );
    } finally {
      setState(() {
        _loading = false;
        _files = null;
      });
      Navigator.of(context).pop();
    }
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async {
        setState(() {
          _loading = false;
          _files = null;
        });
        return true;
      },
      child: Scaffold(
        appBar: ThemedAppBar(
          actions: <Widget>[
            sizedBoxW4,
            if (_loading) const Center(child: ThemedProgress()),
            sizedBoxW4,
            Padding(
              padding: const EdgeInsets.all(10.0),
              child: ThemedTextButton(
                "Select files",
                onPressed: () async {
                  await _getFiles();
                },
              ),
            ),
            sizedBoxW4,
            Padding(
              padding: const EdgeInsets.all(10.0),
              child: ThemedTextButton(
                "Camera",
                onPressed: () async {
                  final ImagePicker _picker = ImagePicker();

                  XFile? file = await showDialog<XFile?>(
                    context: context,
                    builder: (context) => SizedBox(
                      height: 170,
                      child: AlertDialog(
                        content: Row(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              CircleAvatar(
                                radius: 48,
                                child: InkWell(
                                  onTap: () async {
                                    XFile? file = await _picker.pickImage(
                                      source: ImageSource.camera,
                                      imageQuality: 70,
                                    );
                                    return Navigator.of(context).pop(file);
                                  },
                                  child: const Icon(
                                    Icons.image,
                                    size: 45,
                                  ),
                                ),
                              ),
                              sizedBoxW16,
                              CircleAvatar(
                                radius: 48,
                                child: InkWell(
                                  onTap: () async {
                                    XFile? file = await _picker.pickVideo(
                                      source: ImageSource.camera,
                                      maxDuration: const Duration(seconds: 59),
                                    );
                                    return Navigator.of(context).pop(file);
                                  },
                                  child: const Icon(
                                    Icons.video_camera_back_rounded,
                                    size: 45,
                                  ),
                                ),
                              ),
                            ]),
                        actions: [
                          TextButton(
                            onPressed: () {
                              return Navigator.of(context).pop();
                            },
                            child: const Text('Cancel'),
                          ),
                        ],
                      ),
                    ),
                  );
                  if (file != null) {
                    final convertedFile = PlatformFile(
                      name: file.name,
                      size: await file.length(),
                      bytes: await file.readAsBytes(),
                      path: file.path,
                    );
                    setState(() {
                      _files = _files == null
                          ? [convertedFile]
                          : [
                              ..._files!,
                              convertedFile,
                            ];
                    });
                  }
                },
              ),
            ),
            sizedBoxW4,
            if (_files != null && _files!.isNotEmpty)
              Tooltip(
                message: 'Save Files',
                child: Padding(
                  padding: const EdgeInsets.all(4.0),
                  child: IconButton(
                    icon: ThemedIcons.check(),
                    onPressed: () async {
                      await _uploadAndRetutnImageAndUrl(context);
                    },
                  ),
                ),
              ),
            sizedBoxW8,
          ],
        ),
        body: _files != null && _files!.isNotEmpty
            ? LayoutBuilder(builder: (context, constraints) {
                return SizedBox(
                  width: constraints.maxWidth,
                  height: constraints.maxHeight,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        sizedBoxH32,
                        CarouselSlider(
                          items: _files!
                              .asMap()
                              .map(
                                (key, value) => MapEntry(
                                  key,
                                  Stack(
                                    children: [
                                      ImageViewer(
                                        height: constraints.maxHeight - 100,
                                        width: constraints.maxWidth - 40,
                                        data: value.bytes,
                                      ),
                                      Positioned(
                                        top: 17,
                                        left: 17,
                                        child: CircleAvatar(
                                          backgroundColor: Colors.white,
                                          child: Center(
                                            child: IconButton(
                                              onPressed: () {
                                                setState(() {
                                                  _files!.removeAt(key);
                                                });
                                              },
                                              icon: ThemedIcons.clear(
                                                iconStyle: IconStyle(
                                                  color: Colors.red,
                                                ),
                                              ),
                                            ),
                                          ),
                                        ),
                                      ),
                                    ],
                                  ),
                                ),
                              )
                              .values
                              .toList(),
                          carouselController: _controller,
                          options: CarouselOptions(
                            autoPlay: true,
                            // height: constraints.maxHeight - 40,
                            enlargeCenterPage: true,
                            onPageChanged: (index, reason) {
                              setState(() {
                                _current = index;
                              });
                            },
                          ),
                        ),
                        Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: _files!.asMap().entries.map((entry) {
                            return GestureDetector(
                              onTap: () => _controller.animateToPage(entry.key),
                              child: Container(
                                width: 12.0,
                                height: 12.0,
                                margin: const EdgeInsets.symmetric(
                                    vertical: 8.0, horizontal: 4.0),
                                decoration: BoxDecoration(
                                    shape: BoxShape.circle,
                                    color: (Theme.of(context).brightness ==
                                                Brightness.dark
                                            ? Colors.white
                                            : Colors.black)
                                        .withOpacity(
                                            _current == entry.key ? 0.9 : 0.4)),
                              ),
                            );
                          }).toList(),
                        ),
                      ]),
                );
              })
            : SizedBox(
                height: 400,
                width: MediaQuery.of(context).size.width,
                child: const Center(
                  child: ThemedText(
                    "Click select image to start",
                  ),
                ),
              ),
      ),
    );
  }
}
