import "package:collection/collection.dart";
import 'package:verttice_logic/verttice_logic.dart';
import "package:verttice_ui/src/const/box_sizes.dart";
import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";
import 'package:flutter/services.dart';
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/themed_toast.dart';

class DateRangeFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final dynamic value;
  final FormFieldState<dynamic>? state;

  const DateRangeFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.value,
    this.state,
  }) : super(key: key);

  @override
  _DateRangeFormState createState() => _DateRangeFormState();
}

class _DateRangeFormState extends State<DateRangeFormFactory> {
  var _value = <String, int?>{};

  @override
  void initState() {
    super.initState();
    _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
        .cast<String, int?>();
  }

  @override
  void didUpdateWidget(covariant DateRangeFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    final value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value);
    if (!const MapEquality().equals(_value, value)) {
      _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
          .cast<String, int?>();
    }
  }

  @override
  Widget build(BuildContext context) {
    DateTimeRange? initialDate;
    if (_value['startDate'] != null &&
        _value['startDate'] is int &&
        _value['endDate'] != null &&
        _value['endDate'] is int) {
      initialDate = DateTimeRange(
        start: DateTime.fromMillisecondsSinceEpoch(_value['startDate']!),
        end: DateTime.fromMillisecondsSinceEpoch(_value['endDate']!),
      );
    }
    return BorderLabel(
      label: widget.displayName,
      onCopied: () async {
        await Clipboard.setData(
          ClipboardData(
            text:
                "${_value["startDate"] != null && _value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: _value["startDate"]!, time: false) : ""} - ${_value["endDate"] != null && _value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: _value["endDate"]!, time: false) : ""}",
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Copied to clipboard",
        );
      },
      actions: [
        PhysicalModel(
          color: Colors.grey,
          elevation: 5,
          shape: BoxShape.circle,
          child: CircleAvatar(
            backgroundColor: Colors.red,
            child: InkWell(
              child: ThemedIcons.clear(
                iconStyle: IconStyle(
                  color: Colors.white,
                ),
              ),
              onTap: () {
                widget.onChanged(widget.name, null);
                widget.state?.didChange(null);
              },
            ),
          ),
        ),
        sizedBoxW2,
      ],
      child: TextButton.icon(
        icon: ThemedIcons.calendar(),
        label: Text(
          "${_value["startDate"] != null && _value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: _value["startDate"]!, time: false) : ""} - ${_value["endDate"] != null && _value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: _value["endDate"]!, time: false) : ""}",
        ),
        onPressed: () async {
          DateTimeRange? date = await showDateRangePicker(
            context: context,
            firstDate: DateTime(2000),
            lastDate: DateTime(2100),
            initialDateRange: initialDate,
          );
          if (date == null) return;
          final choosenDate = {
            "startDate": date.start.millisecondsSinceEpoch,
            "endDate": DateTime(
              date.end.year,
              date.end.month,
              date.end.day,
              23,
              59,
            ).millisecondsSinceEpoch,
          };
          widget.onChanged(widget.name, choosenDate);
          widget.state?.didChange(choosenDate);
        },
      ),
    );
  }
}
