import "package:flutter/material.dart";
import "package:google_maps_flutter/google_maps_flutter.dart";

class MapCoordinatesPage extends StatefulWidget {
  final double lat;
  final double lng;
  final String placeName;

  const MapCoordinatesPage({
    Key? key,
    required this.lat,
    required this.lng,
    required this.placeName,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _MapCoordinatesBodyState();
}

class _MapCoordinatesBodyState extends State<MapCoordinatesPage> {
  _MapCoordinatesBodyState();
  late double _lat;
  late double _lng;
  late String _placeName;

  Set<Marker> _markers = <Marker>{};

  GoogleMapController? _mapController;

  @override
  void initState() {
    super.initState();

    _lat = widget.lat;
    _lng = widget.lng;
    _placeName = widget.placeName;
    final markers = <Marker>{};
    markers.add(Marker(
      draggable: false,
      markerId: MarkerId("${_lat.toString()}${_lng.toString()}"),
      position: LatLng(_lat, _lng),
      infoWindow: InfoWindow(title: _placeName),
    ));
    _markers = markers;
  }

  @override
  void didUpdateWidget(covariant MapCoordinatesPage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.lat != _lat || widget.lng != _lng) {
      _lat = widget.lat;
      _lng = widget.lng;
      _placeName = widget.placeName;
      final markers = <Marker>{};
      markers.add(Marker(
        draggable: false,
        markerId: MarkerId("${_lat.toString()}${_lng.toString()}"),
        position: LatLng(_lat, _lng),
        infoWindow: InfoWindow(title: _placeName),
      ));
      _markers = markers;
      _mapController?.animateCamera(
        CameraUpdate.newCameraPosition(
          CameraPosition(
            target: LatLng(_lat, _lng),
            zoom: 13.0,
          ),
        ),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: [
        Center(
          child: Padding(
            padding: const EdgeInsets.all(20),
            child: SelectableText(_placeName),
          ),
        ),
        Padding(
          padding: const EdgeInsets.all(18.0),
          child: Center(
            child: SizedBox(
              height: 400,
              width: 600,
              child: GoogleMap(
                onMapCreated: _onMapCreated,
                initialCameraPosition: CameraPosition(
                  target: LatLng(_lat, _lng),
                  zoom: 11.0,
                ),
                markers: _markers,
              ),
            ),
          ),
        ),
      ],
    );
  }

  void _onMapCreated(GoogleMapController controller) async {
    _mapController = controller;
  }
}
