import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/verttice_ui.dart';

class ThemedEditorFilesManeger extends StatefulWidget {
  final List<FileModel> files;
  final String path;
  final Future<void> Function(List<FileModel>) filesDataCallback;
  const ThemedEditorFilesManeger({
    Key? key,
    required this.files,
    required this.path,
    required this.filesDataCallback,
  }) : super(key: key);

  @override
  State<ThemedEditorFilesManeger> createState() =>
      _ThemedEditorFilesManegerState();
}

class _ThemedEditorFilesManegerState extends State<ThemedEditorFilesManeger> {
  late List<FileModel> _files;
  @override
  void initState() {
    super.initState();
    _files = widget.files;
  }

  @override
  void didUpdateWidget(covariant ThemedEditorFilesManeger oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_files, widget.files)) {
      _files = widget.files;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(15.0),
      child: Tooltip(
        message: 'Files manager for editor',
        child: CircleAvatar(
          child: IconButton(
            onPressed: () async {
              await fullPageDialog(
                context: context,
                builder: (BuildContext context) {
                  return EditorFileMangerView(
                    files: _files,
                    path: widget.path,
                    filesDataCallback: widget.filesDataCallback,
                  );
                },
              );
            },
            icon: ThemedIcons.files(),
          ),
        ),
      ),
    );
  }
}

class EditorFileMangerView extends StatelessWidget {
  final List<FileModel> files;
  final String path;
  final Future<void> Function(List<FileModel>) filesDataCallback;
  const EditorFileMangerView({
    Key? key,
    required this.files,
    required this.path,
    required this.filesDataCallback,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: const ThemedAppBar(
        title: 'Editor files manager',
      ),
      body: Padding(
        padding: files.isEmpty
            ? const EdgeInsets.all(50.0)
            : const EdgeInsets.all(20.0),
        child: FileUploadAndViewStorage(
          path: path,
          filesDataCallback: filesDataCallback,
          files: files,
          multi: true,
        ),
      ),
    );
  }
}
